%%%%% solving the optimal growth model by value function iteration

clearvars;
close all;
clc;

%%%%% economic parameters

alpha = 0.30;       %% capital's share in production function
beta  = 1/1.05;     %% time discount factor   
delta = 0.05;       %% depreciation rate
sigma = 1;          %% crra

rho   = (1/beta)-1;
z     = 1.05;


kstar = (alpha*z/(rho+delta))^(1/(1-alpha)); %% steady state
kbar  = (1/delta)^(1/(1-alpha));
ystar = z*kstar^alpha;

cstar = ((rho+(1-alpha)*delta)/(rho+delta))*ystar;

%return

%%%%% numerical parameters

max_iter = 1000;
tol      = 1e-7;
penalty  = 10^16;

nk       = 1001;     %% nodes for k grid
kmin     = tol;      %% essentially 0 
kmax     = 2*kstar; 

curv     = 0.8;        %% controls how far points to 0 (curv = 0 are log-spaced, curv = 1 uniformly spaced)
k        = nodeunif(nk, kmin.^curv, kmax.^curv).^(1/curv);

do_figs = 1;
do_irf  = 1;

%%%%% solve dynamic programming problem

%%%%% reward function

c    = zeros(nk,nk);

for j=1:nk,
    
    c(:,j) = z*(k.^alpha) + (1-delta)*k - k(j);
    
end

%%%%% check consumption non-negative

violations = (c<=0);

c = c.*(c>=0) + eps;

if sigma==1,
    
    u = log(c) - penalty*violations;
    
else
    
    u = (1/(1-sigma))*(c.^(1-sigma) - 1) - penalty*violations;

end

%%%%% initial guess


v        = zeros(nk,1);

v_iterates = zeros(nk,max_iter);
g_iterates = zeros(nk,max_iter);

%%%%% iterate on Bellman operator

for i=1:max_iter,
    
%%%%% RHS of Bellman equation

RHS    = u + beta*kron(ones(nk,1),v');

%%%%% maximize over this to get Tv

[Tv,argmax] = max(RHS,[],2);

%%%%% policy that attains the maximum

g = k(argmax);

v_iterates(:,i) = v;
g_iterates(:,i) = g;

%%%%% check if converged

error = max(abs(Tv-v));

fprintf('%4i %6.2e \n',[i, error]);

if norm(Tv-v,inf)<tol, break, end;

%%%%% if not converged, update guess

v = Tv;

end

last_iter = i;

if error<tol && last_iter<=max_iter,
    
    display('found a fixed point')
      
end

%%%%% consumption policy function

y = z*(k.^alpha);

c = y + (1-delta)*k - g;

i = y - c;




figure(1)
set(gcf,'DefaultLineLineWidth', 3);
subplot(1,2,1)
plot(k,v,'b-')
set(get(gcf,'CurrentAxes'),'FontSize',18,'LineWidth', 1.5)
xlabel('capital stock, $k$' ,'Interpreter','latex')
ylabel('value function, $v(k)$' ,'Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
h = legend('value function, $v(k)$','Location','SouthEast');
set(h,'Interpreter','latex');
axis([0 8 -10 10])

subplot(1,2,2)
plot(k,c,'r-',k,cstar*ones(nk,1),'k--',kstar*ones(nk,1),k,'k--')
set(get(gcf,'CurrentAxes'),'FontSize',18,'LineWidth', 1.5)
xlabel('capital stock, $k$','Interpreter','latex')
ylabel('consumption policy, $c(k)$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
h = legend('consumption policy, $c(k)$','Location','SouthEast');
set(h,'Interpreter','latex');
axis([0 8 0 2.5])

%savefile = 'ps1_savefile_z1_precision.mat'; 
%save(savefile,'z','k','v','c','g','argmax','kstar','cstar','delta');




